"""L3 Core Service product type."""

from orchestrator.domain import SubscriptionModel
from orchestrator.types import SubscriptionLifecycle
from pydantic_forms.types import strEnum

from gso.products.product_blocks.l3_core_service import (
    L3CoreServiceBlock,
    L3CoreServiceBlockInactive,
    L3CoreServiceBlockProvisioning,
)


class L3CoreServiceType(strEnum):
    """Available types of Layer 3 Core Services.

    The core services offered include GÉANT IP for R&E access, and the Internet Access Service.
    """

    GEANT_IP = "GÉANT IP"
    IMPORTED_GEANT_IP = "IMPORTED GÉANT IP"
    IAS = "IAS"
    IMPORTED_IAS = "IMPORTED IAS"
    GWS = "GWS"
    IMPORTED_GWS = "IMPORTED GWS"
    LHCONE = "LHCONE"
    IMPORTED_LHCONE = "IMPORTED LHCONE"
    COPERNICUS = "COPERNICUS"
    IMPORTED_COPERNICUS = "IMPORTED COPERNICUS"


class L3CoreServiceInactive(SubscriptionModel, is_base=True):
    """An inactive L3 Core Service subscription."""

    l3_core_service_type: L3CoreServiceType
    l3_core_service: L3CoreServiceBlockInactive


class L3CoreServiceProvisioning(L3CoreServiceInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A L3 Core Service subscription that's being provisioned."""

    l3_core_service_type: L3CoreServiceType
    l3_core_service: L3CoreServiceBlockProvisioning


class L3CoreService(L3CoreServiceProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """An active L3 Core Service subscription."""

    l3_core_service_type: L3CoreServiceType
    l3_core_service: L3CoreServiceBlock


class ImportedL3CoreServiceInactive(SubscriptionModel, is_base=True):
    """An imported, inactive L3 Core Service subscription."""

    l3_core_service_type: L3CoreServiceType
    l3_core_service: L3CoreServiceBlockInactive


class ImportedL3CoreService(
    ImportedL3CoreServiceInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING, SubscriptionLifecycle.ACTIVE]
):
    """An imported L3 Core Service subscription."""

    l3_core_service_type: L3CoreServiceType
    l3_core_service: L3CoreServiceBlock
